/*****************************
 * ECE 4760: Final Project   *
 * Human Tetris              *
 * Video Arcade Edition      *
 *                           *
 * Adam Papamarcos (aip23)   *
 * Kerran Flanagan (kaf42)   *
 * Cornell University        *
 * Spring 2010               *
 *****************************/

#include <inttypes.h>
#include <avr/io.h>
#include <string.h>
#include <stdio.h>
#include <avr/pgmspace.h>

#include "human_tetris.h"
#include "lab5.h"
#include "Shapes.h"
#include "macros.h"

// ========================== "PRIVATE" VARIABLE DECLARATIONS ==========================


#define MISS_FACTOR 1

#define TOTAL_SHAPES 5




#define BORDER 6 //how much of the screen to ignore for matching, in pixel columns

uint8_t thisValue, lastValue;

uint8_t missed;

// ========================== "PRIVATE" FUNCTION DECLARATIONS ==========================

void displayShape(uint8_t);
void displayPerson();
uint8_t isInShape(uint8_t);
inline uint8_t readShape(uint8_t y, uint8_t x, uint8_t image);

// ========================== FUNCTION DEFINTIONS ==========================

void initTetris() {
	//setup gameplay stuff
	gameTimeMs = 0;
	currentLevel = 0;
	currentShapeStartTimeMs = gameTimeMs;
	clearOutput();
	displayShape(currentLevel);
}

void DoTetrisFrame() {
	//level advancing logic
	if (currentShapeStartTimeMs + SHAPE_TIME_Ms <= gameTimeMs) {
		missed = isInShape(currentLevel);
		if (missed > MISS_FACTOR) {
			GameState = GAMEOVER;
			return;
		} else if (++currentLevel >= TOTAL_SHAPES) {
			GameState = GAMEWIN;
			return;
		}
		currentShapeStartTimeMs = gameTimeMs;
		displayShape(currentLevel);
	}

	//messaging logic
	if ((gameTimeMs - currentShapeStartTimeMs) <= 500) {
		sprintf(buffer, "LEVEL %02u", currentLevel);
		loadFancyString(10, 6, buffer, BLINK);
	} else if ((gameTimeMs - currentShapeStartTimeMs) <= 650) {
		clearOutput();
		displayShape(currentLevel);
	}

	//actual frame
	displayPerson();

	sprintf(buffer, " Time: %02u ", MAX(((currentShapeStartTimeMs + SHAPE_TIME_Ms) - gameTimeMs + 500) / 1000, 0));
	loadFancyString(19, 1, buffer, LBC);

	sprintf(buffer, " Lvl: %02u ", currentLevel);
	loadFancyString(1, 1, buffer, LBC);

	//debug stuff
	//	if (missed > MISS_FACTOR) {
	//		sprintf(buffer, "FAIL %3u", missed);
	//		loadString(11, 1, buffer);
	//	} else {
	//		sprintf(buffer, "WOOT %3u", missed);
	//		loadString(11, 1, buffer);
	//	}


}

//only do this once per shape
void displayShape(uint8_t shapeNumber) {
	//	for (uint8_t i = 0; i < ROWS; i++) {
	//		for (uint8_t j = 0; j < COLS; j++) {
	//			if ((readShape(j, i, shapeNumber) == 1))
	//				Output_Data[i][j] = 192;//192 is grid
	//		}
	//	}

	//load image data into osd display array
	for (uint8_t i = 0; i < ROWS; i += 3) {
		for (uint8_t j = 0; j < COLS; j += 2) {
			topL = readShape(i, j, shapeNumber);
			midL = readShape(i + 1, j, shapeNumber);
			botL = readShape(i + 2, j, shapeNumber);
			topR = readShape(i, j + 1, shapeNumber);
			midR = readShape(i + 1, j + 1, shapeNumber);
			botR = readShape(i + 2, j + 1, shapeNumber);

			//store dots as binary number, each dot is a flag
			//112 is blank (zero in binary) for dots
			if (topL + midL + botL + topR + midR + botR > 3)
				Output_Data[i / 3][j / 2] = WALL_BLOCK;
			//112 + topL + 2 * topR + 4 * midL + 8 * midR + 16 * botL + 32 * botR;
			else
				Output_Data[i / 3][j / 2] = 0;
		}
	}
}

//put person blocks in based on previous value
//remember shape is only written to output once per shape
void displayPerson() {
	//load image data into osd display array
	for (uint8_t i = 0; i < ROWS; i += 3) {
		for (uint8_t j = 0; j < COLS; j += 2) {
			topL = Input_Data[i][j] < Black_Level;
			midL = Input_Data[i + 1][j] < Black_Level;
			botL = Input_Data[i + 2][j] < Black_Level;
			topR = Input_Data[i][j + 1] < Black_Level;
			midR = Input_Data[i + 1][j + 1] < Black_Level;
			botR = Input_Data[i + 2][j + 1] < Black_Level;

			//store dots as binary number, each dot is a flag
			thisValue = (topL + midL + botL + topR + midR + botR > 3);
			lastValue = Output_Data[i / 3][j / 2];

			//filter over time
			if (lastValue == BLANK) {
				Output_Data[i / 3][j / 2] = (thisValue) ? PERSON_BLOCK_IN + 8 : BLANK;
			} else if (lastValue >= PERSON_BLOCK_IN && lastValue < PERSON_BLOCK_IN + 16) {
				Output_Data[i / 3][j / 2] = (thisValue) ? MIN(PERSON_BLOCK_IN + 15, lastValue+2)
						: MAX(PERSON_BLOCK_IN, lastValue-2);
			} else if (lastValue == WALL_BLOCK) {
				Output_Data[i / 3][j / 2] = (thisValue) ? PERSON_BLOCK_OUT + 8 : WALL_BLOCK;
			} else if (lastValue >= PERSON_BLOCK_OUT && lastValue < PERSON_BLOCK_OUT + 16) {
				Output_Data[i / 3][j / 2] = (thisValue) ? MIN(PERSON_BLOCK_OUT + 15, lastValue+2)
						: MAX(PERSON_BLOCK_OUT, lastValue-2);
			}

		}
	}
}

//call after display person
uint8_t isInShape(uint8_t shapeNumber) {
	uint8_t missed = 0;
	for (uint8_t i = 0; i < ROWS; i += 3) {
		for (uint8_t j = 0 + BORDER; j < COLS - BORDER; j += 2) {
			//			// High res approach
			//			if (readShape(j, i, shapeNumber) == 1) {
			//				topL = Input_Data[i][j] < Black_Level;
			//				midL = Input_Data[i + 1][j] < Black_Level;
			//				botL = Input_Data[i + 2][j] < Black_Level;
			//				topR = Input_Data[i][j + 1] < Black_Level;
			//				midR = Input_Data[i + 1][j + 1] < Black_Level;
			//				botR = Input_Data[i + 2][j + 1] < Black_Level;
			//
			//				//store dots as binary number, each dot is a flag
			//				if (topL + midL + botL + topR + midR + botR > 3)
			//					missed++;
			//			}

			//low res filtered approach
			if (Output_Data[i / 3][j / 2] >= (PERSON_BLOCK_OUT + 8) && Output_Data[i / 3][j / 2] < (PERSON_BLOCK_OUT
					+ 16))
				missed++;
		}

	}

	return missed;
}

inline uint8_t readShape(uint8_t y, uint8_t x, uint8_t image) {
	//uint8_t b = ((uint8_t) pgm_read_byte(&Shapes[image][(y*30+x)/8]));
	//return (b >> ((y*30+x)%8)) & 0x01;
	return (((uint8_t) pgm_read_byte(&Shapes[image][(y*COLS+x)/8])) >> ((y * COLS + x) % 8)) & 0x01;
}
