/*****************************
 * ECE 4760: Final Project   *
 * Human Tetris              *
 * Video Arcade Edition      *
 *                           *
 * Adam Papamarcos (aip23)   *
 * Kerran Flanagan (kaf42)   *
 * Cornell University        *
 * Spring 2010               *
 *****************************/

/****************************************************************************
 *   Copyright (C) 2009 by Claas Anders "CaScAdE" Rathje                    *
 *   admiralcascade@gmail.com                                               *
 *   Project-URL: http://www.mylifesucks.de/oss/c-osd/                      *
 *                                                                          *
 *   This program is free software; you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by   *
 *   the Free Software Foundation; either version 2 of the License.         *
 *                                                                          *
 *   This program is distributed in the hope that it will be useful,        *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for more details.                           *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to the                          *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 *                                                                          *
 *   Extended and modified for ATmega644 in 2010 by                         *
 *   Kerran Flanagan and Adam Papamarcos.                                   *
 ****************************************************************************/

#ifndef SPI_H_
#define SPI_H_

/* ##########################################################################
 * Software SPI to communicate with MAX7456
 * ##########################################################################*/

#define	ALLCHARSDEBUG	0

//must set both of these to 1 to flash all chars
#define	WRITECHARS		0
#define WRITEALLCHARS  	0

#define UART_DEBUG	0

#define MAX_CS_HIGH             PORTB |=  (1 << PB4);
#define MAX_CS_LOW              PORTB &= ~(1 << PB4);
#define MAX_SDIN_HIGH           PORTB |=  (1 << PB5);
#define MAX_SDIN_LOW            PORTB &= ~(1 << PB5);
#define MAX_SCLK_HIGH           PORTB |=  (1 << PB7);
#define MAX_SCLK_LOW            PORTB &= ~(1 << PB7);
#define MAX_RESET_HIGH          PORTB |=  (1 << PB3);
#define MAX_RESET_LOW           PORTB &= ~(1 << PB3);

#define MAX_SD_OUT				((PORTB & ~(1 << PB6)) >> 6)

/* ##########################################################################
 * MAX7456 SPI & Display stuff
 * ##########################################################################*/

/**
 * Send a byte through SPI
 */
void spi_send(uint8_t);

/**
 *  Send <byte> to <address> of MAX7456
 */
void spi_send_byte(uint8_t, uint8_t);

/**
 *  Read a <byte> from <address> of MAX7456
 */
unsigned char spi_read_byte(uint8_t address);

/**
 *  write a <character> to <address> of MAX7456 display memory
 */
void write_char(uint16_t, char);

/**
 *  write a character <attribute> to <address> of MAX7456 display memory
 */
void write_char_att(uint16_t, char);
/**
 *  write a <character> at <x>/<y> to MAX7456 display memory
 */
void write_char_xy(uint8_t, uint8_t, char);
/**
 *  write a  character <attribute> at <x>/<y> to MAX7456 display memory
 */
void write_char_att_xy(uint8_t, uint8_t, char attribute);

/**
 *  clear display by writing blank characters all over it
 */
void clear(void);

/**
 * remap an ascii char to its location in the MAX7456 character memory
 */
char remap_ascii_char(char c);

/**
 *  write an ascii <character> to <address> of MAX7456 display memory
 */
void write_ascii_char(uint16_t, char);

/**
 *  write an ascii <string> at <x>/<y> to MAX7456 display memory
 */
void write_ascii_string(uint8_t, uint8_t, char*);


/**
 * for testing write all chars to screen
 */
void write_all_chars();

/**
 * Setup SPI for talking to OSD.
 */
void init_SPI();



#endif /* SPI_H_ */
